@@ -1,7 +1,7 @@
 /*
  * Interface to libclamunrar
  *
- * Copyright (C) 2013-2019 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ * Copyright (C) 2013-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  * Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  * Authors: Trog, Torok Edvin, Tomasz Kojm, Micah Snyder
@@ -54,8 +54,7 @@ typedef enum cl_unrar_error_tag {
     UNRAR_EOPEN
 } cl_unrar_error_t;
 
-typedef struct unrar_metadata_tag
-{
+typedef struct unrar_metadata_tag {
     uint64_t pack_size;
     uint64_t unpack_size;
     char *filename;
@@ -67,7 +66,7 @@ typedef struct unrar_metadata_tag
 
 cl_unrar_error_t unrar_open(const char *filename, void **hArchive, char **comment, uint32_t *comment_size, uint8_t debug_flag);
 cl_unrar_error_t unrar_peek_file_header(void *hArchive, unrar_metadata_t *file_metadata);
-cl_unrar_error_t unrar_extract_file(void* hArchive, const char* destPath, char *outputBuffer);
+cl_unrar_error_t unrar_extract_file(void *hArchive, const char *destPath, char *outputBuffer);
 cl_unrar_error_t unrar_skip_file(void *hArchive);
 void unrar_close(void *hArchive);
 
